import { NodeOptions, HighlightContext } from '@highlight-run/node';
import { ResourceAttributes } from '@opentelemetry/resources/build/src/types';
import { ExecutionContext } from '@cloudflare/workers-types';
import { WorkersSDK } from '@highlight-run/opentelemetry-sdk-workers';
import { Attributes } from '@opentelemetry/api';
import { IncomingHttpHeaders } from 'http';
import { NextRequest, NextFetchEvent } from 'next/server';

type HighlightEnv = NodeOptions;
declare interface Metric {
    name: string;
    value: number;
    tags?: {
        name: string;
        value: string;
    }[];
}
type ExtendedExecutionContext = ExecutionContext & {
    __waitUntilTimer?: ReturnType<typeof setInterval>;
    __waitUntilPromises?: Promise<void>[];
    waitUntilFinished?: () => Promise<void>;
};
interface HighlightInterface {
    init: (options: NodeOptions) => void;
    initEdge: (request: Request, env: HighlightEnv, ctx: ExtendedExecutionContext, serviceName?: string) => WorkersSDK;
    isInitialized: () => boolean;
    metrics: (metrics: Metric[]) => void;
    parseHeaders: (headers: IncomingHttpHeaders) => HighlightContext;
    runWithHeaders: <T>(headers: IncomingHttpHeaders, cb: () => T) => T;
    setHeaders: (headers: IncomingHttpHeaders) => void;
    consumeError: (error: Error, secureSessionId?: string, requestId?: string, metadata?: Attributes) => void;
    consumeAndFlush: (error: Error, secureSessionId?: string, requestId?: string, metadata?: Attributes) => void;
    sendResponse: (response: Response) => void;
    setAttributes: (attributes: ResourceAttributes) => void;
    recordMetric: (secureSessionId: string, name: string, value: number, requestId: string, tags?: {
        name: string;
        value: string;
    }[]) => void;
    stop: () => Promise<void>;
    flush: () => Promise<void>;
}

type EdgeHandler = (request: NextRequest, event: NextFetchEvent) => Promise<Response>;

export { EdgeHandler as E, HighlightInterface as H, HighlightEnv as a, ExtendedExecutionContext as b };
